/******************************************************************************
 *
 * Filename: if.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  4th March 2002
 *
 * Purpose:  Include file for if instructions.
 *
 * (c) Copyright R.H.Thornburrow, 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

inline void if_acmpeq( Frame* fp, u2 incr )
{
	ObjRef* value1;
	ObjRef* value2;

	INSTRUCTION( "%p: if_acmpeq %d\n", fp->pc, (int16) incr );

	POP_OBJREF( fp, value2 );
	POP_OBJREF( fp, value1 );
	
	if( value1 == value2 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void if_acmpne( Frame* fp, u2 incr )
{
	ObjRef* value1;
	ObjRef* value2;

	INSTRUCTION( "%p: if_acmpne %d\n", fp->pc, (int16) incr );

	POP_OBJREF( fp, value2 );
	POP_OBJREF( fp, value1 );

	if( value1 != value2 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void if_icmpeq( Frame* fp, u2 incr )
{
	int32 value1, value2;

	INSTRUCTION( "%p: if_icmpeq %d\n", fp->pc, (int16) incr );

	POP_INT( fp, value2 );
	POP_INT( fp, value1 );
	
	if( value1 == value2 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void if_icmpge( Frame* fp, u2 incr )
{
	int32 value1, value2;

	INSTRUCTION( "%p: if_icmpge %d\n", fp->pc, incr );

	POP_INT( fp, value2 );
	POP_INT( fp, value1 );
	
	if( value1 >= value2 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void if_icmpgt( Frame* fp, u2 incr )
{
	int32 value1, value2;

	INSTRUCTION( "%p: if_icmpgt %d\n", fp->pc, (int16) incr );

	POP_INT( fp, value2 );
	POP_INT( fp, value1 );
	
	if( value1 > value2 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void if_icmple( Frame* fp, u2 incr )
{
	int32 value1, value2;

	INSTRUCTION( "%p: if_icmple %d\n", fp->pc, (int16) incr );

	POP_INT( fp, value2 );
	POP_INT( fp, value1 );
	
	if( value1 <= value2 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void if_icmplt( Frame* fp, u2 incr )
{
	int32 value1, value2;

	INSTRUCTION( "%p: if_icmplt %d\n", fp->pc, (int16) incr );

	POP_INT( fp, value2 );
	POP_INT( fp, value1 );
	
	if( value1 < value2 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void if_icmpne( Frame* fp, u2 incr )
{
	int32 value1, value2;

	INSTRUCTION( "%p: if_icmpne %d\n", fp->pc, (int16) incr );

	POP_INT( fp, value2 );
	POP_INT( fp, value1 );
	
	if( value1 != value2 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void ifeq( Frame* fp, u2 incr )
{
	int32 value;

	INSTRUCTION( "%p: ifeq %d\n", fp->pc, (int16) incr );

	POP_INT( fp, value );
	
	if( value == 0 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void ifge( Frame* fp, u2 incr )
{
	int32 value;

	INSTRUCTION( "%p: ifge %d\n", fp->pc, (int16) incr );

	POP_INT( fp, value );
	
	if( value >= 0 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void ifgt( Frame* fp, u2 incr )
{
	int32 value;

	INSTRUCTION( "%p: ifgt %d\n", fp->pc, (int16) incr );

	POP_INT( fp, value );
	
	if( value > 0 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void ifle( Frame* fp, u2 incr )
{
	int32 value;

	INSTRUCTION( "%p: ifle %d\n", fp->pc, (int16) incr );

	POP_INT( fp, value );
	
	if( value <= 0 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void iflt( Frame* fp, u2 incr )
{
	int32 value;

	INSTRUCTION( "%p: iflt %d\n", fp->pc, (int16) incr );

	POP_INT( fp, value );
	
	if( value < 0 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void ifne( Frame* fp, u2 incr )
{
	int32 value;

	INSTRUCTION( "%p: ifne %d\n", fp->pc, (int16) incr );

	POP_INT( fp, value );
	
	if( value != 0 )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void ifnonnull( Frame* fp, u2 incr )
{
	ObjRef* value;

	INSTRUCTION( "%p: ifnonnull %d\n", fp->pc, (int16) incr );

	POP_OBJREF( fp, value );
	
	if( value )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}


inline void ifnull( Frame* fp, u2 incr )
{
	ObjRef* value;

	INSTRUCTION( "%p: ifnull %d\n", fp->pc, (int16) incr );

	POP_OBJREF( fp, value );
	
	if( !value )
		fp->pc += (int16) incr;
	else
		fp->pc += 3;
}
