/******************************************************************************
 *
 * Filename: object.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  4th March 2002
 *
 * Purpose:  Include file for object ref instructions.
 *
 * (c) Copyright R.H.Thornburrow, 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

int aaload( Stack* stp )
{
	Frame *fp = stp->fp;
	int32 index;
	ObjRef* array;
	ObjRef** data;

	INSTRUCTION( "%p: aaload\n", fp->pc );

	POP_INT( fp, index );
	POP_OBJREF( fp, array );

	if( !array )
		return newCompleteException( stp, NullPointerException, "aaload" );

	if( (index < 0) || (index >= (int32) array->length) )
	{
		char buf[32];
		sprintf( buf, "%d", (int) index );
		return newCompleteException( stp, ArrayIndexOutOfBoundsException, buf );
	}

	if( (array->cf != NULL) || (array->type != T_OBJREF) || (array->data == NULL) )
		return newCompleteException( stp, InternalError, "Bad object reference array" );

	data = (ObjRef**) array->data;
	
	/*if( data[index] )
	{
		char buf[255];
		sprintClassName( buf, data[index]->cf, data[index]->cf->this_class );
		printf( "aaload pushing 0x%08x of type %s\n", data[index], buf );
		printf( "aaload pushing 0x%08x of type %d\n", data[index], data[index]->type );
	}
	else
		printf( "aaload pushing null\n" );*/

	PUSH_OBJREF( fp, data[index] );

	return 0;
}


int aastore( Stack* stp )
{
	Frame *fp = stp->fp;
	ObjRef* value;
	int32 index;
	ObjRef* array;
	ObjRef** data;

	INSTRUCTION( "%p: aastore\n", fp->pc );

	POP_OBJREF( fp, value );
	POP_INT( fp, index );
	POP_OBJREF( fp, array );

	/*if( value )
	{
		char buf[255];
		sprintClassName( buf, value->cf, value->cf->this_class );
		printf( "aastore popped 0x%08x of type %s\n", value, buf );
	}
	else
		printf( "aastore popped null\n" );*/

	if( !array )
		return newCompleteException( stp, NullPointerException, "aastore" );

	if( (index < 0) || (index >= (int32) array->length) )
	{
		char buf[32];
		sprintf( buf, "%d", (int) index );
		return newCompleteException( stp, ArrayIndexOutOfBoundsException, buf );
	}

	if( (array->cf != NULL) || (array->type != T_OBJREF) || (array->data == NULL) )
		return newCompleteException( stp, InternalError, "Bad object reference array" );

	data = (ObjRef**) array->data;
	data[index] = value;
	return 0;
}


inline void aconst_null( Frame* fp )
{
	INSTRUCTION( "%p: aconst_null\n", fp->pc );
	PUSH_OBJREF( fp, NULL );
}


inline void aload_n( Frame* fp, u2 index )
{
	INSTRUCTION( "%p: aload_%d\n", fp->pc, index );
	PUSH_LOBJREF( fp, index );
}


int areturn( Stack *stp )
{
	Frame* fp = stp->fp;
	ObjRef* objref;
	int stopOnPop = stp->fp->stopOnPop;

	INSTRUCTION( "%p: areturn\n", fp->pc );

	POP_OBJREF( fp, objref );
	stp->fp = fp->next;
	freeStackFrame( fp, fp->mi );
	fp = stp->fp;

	if( fp )
	{
		fp->prev = NULL;
		PUSH_OBJREF( fp, objref );
	}

#ifdef _DEBUG
	if( fp && method_debug )
	{
		char buf[255];
		sprintMethodInfo( buf, fp->mi );
		printf( "Returned to method %s\n", buf );
	}
#endif /* _DEBUG */

	if( callstack )
		(*callstack)( fp, 0 );

	return ( (!fp || stopOnPop) ? 1 : 0 );
}


inline void astore_n( Frame* fp, u2 index )
{
	u1 type;
	u4 value;

	INSTRUCTION( "%p: astore_%d\n", fp->pc, index );

	POP_ANY_TYPE( fp, value, type );

	if( (type != T_OBJREF) && (type != T_RETADDR) )
	{
		DEBUG( printf( "ERROR: astore_n, stack corrupt (type %d != %d)\n", fp->tlocal[index], type ); )
		halt = 1;
	}

	fp->local[index] = value;
	fp->tlocal[index] = type;
}
