/******************************************************************************
 *
 * Filename: short.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  4th March 2002
 *
 * Purpose:  Include file for short instructions.
 *
 * (c) Copyright R.H.Thornburrow, 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

int saload( Stack* stp )
{
	Frame *fp = stp->fp;
	int32 index;
	ObjRef* array;
	u2* data;

	INSTRUCTION( "%p: saload\n", fp->pc );

	POP_INT( fp, index );
	POP_OBJREF( fp, array );

	if( !array )
		return newCompleteException( stp, NullPointerException, "saload" );

	if( (index < 0) || (index >= (int32) array->length) )
	{
		char buf[32];
		sprintf( buf, "%d", (int) index );
		return newCompleteException( stp, ArrayIndexOutOfBoundsException, buf );
	}

	if( (array->cf != NULL) || (array->type != T_SHORT) || (array->data == NULL) )
		return newCompleteException( stp, InternalError, "Bad short[] object reference" );

	data = (u2*) array->data;

	PUSH_INT( fp, data[index] );

	ARITHMETIC_DEBUG( printf( "%p: saload: 0x%p [ %d ] => 0x%04x\n", fp->pc, data, (int) index, (u2) data[index] ); )
	return 0;
}


int sastore( Stack* stp )
{
	Frame *fp = stp->fp;
	u4 value;
	int32 index;
	ObjRef* array;
	u2* data;

	INSTRUCTION( "%p: sastore\n", fp->pc );

	POP_INT( fp, value );
	POP_INT( fp, index );
	POP_OBJREF( fp, array );

	if( !array )
		return newCompleteException( stp, NullPointerException, "sastore" );

	if( (index < 0) || (index >= (int32) array->length) )
	{
		char buf[32];
		sprintf( buf, "%d", (int) index );
		return newCompleteException( stp, ArrayIndexOutOfBoundsException, buf );
	}

	if( (array->cf != NULL) || (array->type != T_SHORT) || (array->data == NULL) )
		return newCompleteException( stp, InternalError, "Bad short[] object reference" );

	data = (u2*) array->data;

	data[index] = (u2) value;

	ARITHMETIC_DEBUG( printf( "%p: sastore: 0x%p [ %d ] <= 0x%04x\n", fp->pc, data, (int) index, (u2) value ); )
	return 0;
}


inline void sipush( Frame* fp, u2 s )
{
    int32 value = (s > 32767) ? (int32) (s - 65536) : (int32) s;

	INSTRUCTION( "%p: sipush %d\n", fp->pc, (int) value );
	PUSH_INT( fp, value );
}
