/******************************************************************************
 *
 * Filename: util.h
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  19th January 1998
 *
 * Purpose:  Class construction utilities.
 *
 * (c) Copyright R.H.Thornburrow, 1998
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#ifndef _util_h

#define _util_h

#ifdef __cplusplus
extern "C" {
#endif

#include "engine.h"
#include "types.h"

extern Frame* newStackFrame( method_info* mi, Frame* next );
extern void freeStackFrame( Frame* fp, method_info* mi );

extern u2 countUtf8( const u1* utf8, u2 utf8len );
extern void w2cstrncpy( char* dst, const u2* src, int len );

extern ObjRef* newJavaLangClass( ClassFile* cf );
extern ObjRef* newJavaLangStringFromBytes( const char* bytes, int length );
extern ObjRef* newJavaLangStringFromUtf8( const struct _CONSTANT_Utf8_info* utf8 );
extern ObjRef* newJavaLangThread( const char* name, ObjRef* group );
extern ObjRef* newJavaIoFileDescriptor( int fd );
extern ObjRef* newJavaIoObjectStreamField( ObjRef* name, u1 type, u4 offset, ObjRef* typeString );

extern u4 sizeOfJavaType( u1 type );

extern ObjRef* newObjectRefAndObject( ClassFile* cf );
extern ObjRef* newObjectRefAndArray( u1 type, int32 length );

extern ClassFile* getClass( Stack* stp, const char* classname );
extern int checkType( ClassFile* cf, const char* classname );
extern char* extractFromJavaLangString( char* buf, int buflen, ObjRef* str );

extern int newUnsatisfiedLinkError( Stack* stp, ClassFile* cf, u2 index );
extern int newUnsatisfiedLinkErrorIface( Stack* stp, ClassFile* cf, u2 index );
extern int newNoSuchMethodError( Stack* stp, ClassFile* cf, u2 index );
extern int newNoSuchMethodErrorIface( Stack* stp, ClassFile* cf, u2 index );
extern int newNoSuchFieldError( Stack* stp, ClassFile* cf, u2 index );

extern const char* exceptionKindToString( ExceptionKind kind );
extern void whereAreWe( Frame* baseFrame );
extern ObjRef* createMultiObjrefArray( int dimension, int32* length, u1 type );
extern void addToList( ObjRef* objref, ObjRefList** pool );
extern void addMemoryBlock( void* block, size_t size );

extern int findFreeFD();

extern int32 getFieldOffset( ClassFile* cf, const char* name, const char* desc );

#ifdef __cplusplus
}
#endif


#endif
