/******************************************************************************
 *
 * Filename: zipfile.h
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  10th May 2003
 *
 * Purpose:  Common definitions for ZIP files.
 *
 * (c) Copyright R.H.Thornburrow, 1998-2003
 *
 * Based on Paul Katz's public domain specification for the PKZIP file format.
 * No proprietry code has been referenced in developing this module.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#ifndef __zipfile_h

#define __zipfile_h


#include "types.h"


typedef enum
{
	zm_none = 0,     zm_shrunk,
	zm_reduced_1,    zm_reduced_2,    zm_reduced_3,    zm_reduced_4,
	zm_imploded_2_4, zm_imploded_3_4, zm_imploded_2_8, zm_imploded_3_8
} ZipMethod;


#define CENTRAL_DIR_MAGIC 0x02014b50
#define LOCAL_FILE_MAGIC  0x04034b50
#define END_CENTRAL_MAGIC 0x06054b50

#define ZIP_MAX_PATH	  256
#define ZIP_MAX_CMNT	  256


typedef struct _ZipEndCentralDir
{
	u2		zipedsk;	/* Number of this disk */
	u2		zipecen;	/* Number of disk with start central dir */
	u2		zipenum;	/* Total number of entries in central dir on this disk */ 
	u2		zipecenn;	/* total number entries in central dir */
	u4		zipecsz;	/* Size of the central directory */
	u4		zipeofst;	/* Offset of start of central directory with
						   respect to the starting disk number */
	u2		zipecoml;	/* zipfile comment length */
	char	zipecom[ZIP_MAX_CMNT]; /* zipfile comment */
} ZipEndCentralDir;


typedef struct _ZipCentralDir
{
	u1     zipcver;		/* Version made by */
	u1     zipcos;      /* Host operating system */
	u1     zipcvxt;     /* Version needed to extract */
	u1     zipcexos;    /* O/S of version needed for extraction */
	u2     zipcflg;     /* General purpose bit flag */
	u2     zipcmthd;    /* Compression method */
	u2     zipctim;     /* Last mod file time (MS-DOS) */
	u2     zipcdat;     /* Last mod file date (MS-DOS) */
	u4     zipccrc;     /* CRC-32 */
	u4     zipcsiz;     /* Compressed size */
	u4     zipcunc;     /* Uncompressed size */
	u2     zipcfnl;     /* Filename length */
	u2     zipcxtl;     /* Extra field length */
	u2     zipccml;     /* File comment length */
	u2     zipdsk;      /* Disk number start */
	u2     zipint;      /* Internal file attributes */
	u4     zipext;      /* External file attributes, host system dependent */
	u4     zipofst;     /* Relative offset of local header from the start of
						   the first disk on which this file appears */
	char   zipcfn[ZIP_MAX_PATH]; /* Filename or path - should not contain a drive or 
						   device letter, or a leading slash. All slashes
						   should be forward slashes '/' */
	/*u1     zipcxtr[256];*/ /* extra field */
	/*char   zipcom[256];*/ /* file comment */
} ZipCentralDir;


typedef struct _ZipLocalFile
{
	u2     zipver;       /* Version needed to extract */
	u2     zipgenflg;    /* General purpose bit flag */
	u2     zipmthd;      /* Compression method */
	u2     ziptime;      /* Last mod file time (MS-DOS) */
	u2     zipdate;      /* Last mod file date (MS-DOS) */
	u4     zipcrc;       /* CRC-32 */
	u4     zipsize;      /* Compressed size */
	u4     zipuncmp;     /* Uncompressed size */
	u2     zipfnln;      /* Filename length */
	u2     zipxtraln;    /* Extra field length */
	char   zipname[ZIP_MAX_PATH]; /* filename */
	/*u1     zipxtra[256];*/ /* extra field */
} ZipLocalFile;


typedef struct _ZipCacheEntry
{
	u4	   zipofst;		/* Relative offset of local header from the start of
						   the first disk on which this file appears */
} ZipCacheEntry;


#ifdef __cplusplus
extern "C" {
#endif

extern ZipCacheEntry* newZipCacheEntry( u4 zipofst );
extern void readWord( void* d, FILE* f );
extern void readDWord( void* d, FILE* f );
extern int skipN( FILE* f, u4 count );

extern int32 findCentralDirectory( FILE *f );
extern int readCentralDirectory( ZipCentralDir *record, FILE *f );
extern int readLocalFile( ZipLocalFile* local, FILE *f );

#ifdef __cplusplus
}
#endif


#endif /* __zipfile_h */
