/******************************************************************************
 *
 * Filename: java_lang_Class.h
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  19th January 1998
 *
 * Purpose:  Native method implementations for java.lang.Class.
 *
 * (c) Copyright R.H.Thornburrow, 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#ifndef _java_lang_Class_h

#define _java_lang_Class_h


#include "../engine/jni.h"

#include "../engine/classfile.h"


#ifdef __cplusplus
extern "C" {
#endif

#if JDK_VERSION >= 140
jboolean Java_java_lang_Class_desiredAssertionStatus0( JNIEnv* env, jclass staticself, jclass aClass );
#endif /* JDK 1.4 */

#if JDK_VERSION < 140
extern jclass   Java_java_lang_Class_forName( JNIEnv* env, jclass staticself, jstring classname );
extern jobject  Java_java_lang_Class_getClassLoader( JNIEnv* env, jobject self );
#else
extern jclass   Java_java_lang_Class_forName0(
	JNIEnv* env, jclass staticself, jstring str, jboolean aBoolean, jobject aClassLoader );
extern jobject  Java_java_lang_Class_getClassLoader0( JNIEnv* env, jobject self );
#endif /* JDK_VERSION 1.4.0 */


#if JDK_VERSION < 140
extern jobject  Java_java_lang_Class_getConstructors0( JNIEnv* env, jobject self, jint privateFlag );
extern jobject Java_java_lang_Class_getConstructor0( JNIEnv* env, jobject self, jobject paramTypes, jint privateFlag );
extern jobject  Java_java_lang_Class_getFields0( JNIEnv* env, jobject self, jint privateFlag );
extern jobject  Java_java_lang_Class_getField0( JNIEnv* env, jobject self, jobject fieldName, jint privateFlag );
#else
extern jobject  Java_java_lang_Class_getDeclaredConstructors0( JNIEnv* env, jobject self, jboolean publicOnly );
extern jobject  Java_java_lang_Class_getDeclaredFields0( JNIEnv* env, jobject self, jboolean publicOnly );
#endif /* JDK 1.4 */

extern jobject  Java_java_lang_Class_getInterfaces( JNIEnv *env, jobject self );
extern jint     Java_java_lang_Class_getModifiers( JNIEnv *env, jobject self );
extern jstring  Java_java_lang_Class_getName( JNIEnv* env, jobject self );
extern jclass   Java_java_lang_Class_getPrimitiveClass( JNIEnv* stp, jclass cls, jstring name );
extern jclass   Java_java_lang_Class_getSuperclass( JNIEnv* env, jobject self );
extern jboolean Java_java_lang_Class_isArray( JNIEnv* env, jobject self );
extern jboolean Java_java_lang_Class_isAssignableFrom( JNIEnv* env, jobject self, jclass cls );
extern jboolean Java_java_lang_Class_isInterface( JNIEnv* env, jobject self );
extern jboolean Java_java_lang_Class_isPrimitive( JNIEnv* env, jobject self );

#if JDK_VERSION >= 140
extern void     Java_java_lang_Class_registerNatives( JNIEnv* env, jclass staticself );
#endif /* JDK_VERSION 1.4.0 */


/* Internal functions */
extern u1 getPrimitiveTypeForClass( jclass clazz );
extern jclass getClassForArrayType( const char* desc );
extern int isArrayType( jclass clazz );
extern jobject descriptorToClassArray( JNIEnv* env, ClassFile* cf, u2 index );


#ifdef __cplusplus
}
#endif

#endif /* _java_lang_Class_h */
