/******************************************************************************
 *
 * Filename: java_lang_Math.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  25th March 2002
 *
 * Purpose:  Native method implementations for java.lang.Math.
 *
 * (c) Copyright R.H.Thornburrow, 2002-3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <math.h>

#include "../engine/config.h"

#include "java_lang_Math.h"


/* JDK 1.1.7A (all non-native in 1.4 and natives moved tp java.lang.StrictMath)
 #  public static native double sin(double a);
 #  public static native double cos(double a);
 #  public static native double tan(double a);
 #  public static native double asin(double a);
 #  public static native double acos(double a); 
 #  public static native double atan(double a);
 #  public static native double exp(double a);
+#  public static native double log(double a);
 #  public static native double sqrt(double a);
 #  public static native double IEEEremainder(double f1, double f2);
 #  public static native double ceil(double a);
+#  public static native double floor(double a);
 #  public static native double rint(double a);
 #  public static native double atan2(double a, double b);
 #  public static native double pow(double a, double b);
*/

#if JDK_VERSION < 140

/**
 * Returns the largest integer (expressed as a double) that is not greater than
 * 'a'.
 *
 * @since  JDK1.0 (not native in JDK1.4 - moved to java.lang.StrictMath)
 */
jdouble Java_java_lang_Math_floor( JNIEnv* env, jclass staticself, jdouble a )
{
	return floor( a );
}


/**
 * Returns the natural logarithm (ln or log to the base e) of 'a'. 'a' must be
 * a number greater than 0.0.
 *
 * @since   JDK1.0 (not native in JDK1.4 - moved to java.lang.StrictMath)
 */
jdouble Java_java_lang_Math_log( JNIEnv* env, jclass staticself, jdouble a )
{
	return log( a );
}

#endif /* JDK 1.1 */
