/******************************************************************************
 *
 * Filename: java_lang_Runtime.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  25th March 2002
 *
 * Purpose:  Native method implementations for java.lang.Runtime.
 *
 * (c) Copyright R.H.Thornburrow, 2002-3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>

#include "../engine/config.h"
#include "../engine/util.h"
#include "../engine/globals.h"

#include "java_lang_Runtime.h"


/* JDK 1.1.7A & 1.4	
+#   private native synchronized java.lang.String initializeLinkerInternal();
+#   private native java.lang.String buildLibName(String, String);
+#   private native int loadFileInternal(java.lang.String);
+#   private native synchronized void exitInternal(int);
     private native java.lang.Process execInternal(String[], String[]);
 1.4 private native java.lang.Process execInternal(String[], String[], String)
		 throws java.io.IOException;
 1.4 public native int availableProcessors();
     public native long freeMemory();
     public native long totalMemory();
     public native long maxMemory();
     public native void gc();
 #   private static native void runFinalizersOnExit0(boolean);
 #   public native void runFinalization();
 1.4 private static native void runFinalization0();
+    public native void traceInstructions(boolean);
+    public native void traceMethodCalls(boolean);

	 +   - Implemented
	 -   - Dummied
	 #   - In 1.1, but not in 1.4
	 1.4 - In 1.4 (may be in 1.2)
*/

#if JDK_VERSION < 140
/* private native String buildLibName(String pathname, String filename); */
jobject Java_java_lang_Runtime_buildLibName( JNIEnv* env, jobject self, jobject pathstr, jobject filestr )
{
	char filename[255];
	char pathname[255];
	ObjRef *str;

	if( extractFromJavaLangString( filename, 255, filestr )
		&& extractFromJavaLangString( pathname, 255, pathstr ) )
	{
		char buffer[512];

		sprintf( buffer, "%s%s%s.%s", pathname, FILE_SEPARATOR, filename, LIBRARY_EXTENSION );
		str = newJavaLangStringFromBytes( buffer, strlen( buffer ) );
		addToList( str, &objectPool );
		return str;
	}

	env->retcode = newCompleteException( env, InternalError, "Can't get value of java.lang.String" );
	return NULL;
}


/**
 * Helper for exit
 *
 * //__SYMC__
 * private native synchronized void exitInternal(int status);
 * //  private native void exitInternal(int status);
 * //__SYMC__
 */
void Java_java_lang_Runtime_exitInternal( JNIEnv* env, jobject self, jint retcode )
{
	printf( "WARNING: Runtime.exitInternal( %d ) called.\n", (int) retcode );
	env->retcode = 1; /* exit */
}


/**
 * Initializes the linker and returns the search path for shared libraries.
 *
 * private synchronized native String initializeLinkerInternal();
 */
jobject Java_java_lang_Runtime_initializeLinkerInternal( JNIEnv* env, jobject self )
{
	ObjRef* str = newJavaLangStringFromBytes( ".", 1 );
	addToList( str, &objectPool );
	return str;
}


/**
 * Helper for load and loadLibrary
 *
 * Returns: -1 => OutOfMemory
 *           0 => UnsatisfiedLinkError
 *
 * private native int loadFileInternal(String filename);
 */
jint Java_java_lang_Runtime_loadFileInternal( JNIEnv* env, jobject self, jobject str )
{
	char filename[255];

	if( extractFromJavaLangString( filename, 255, str ) )
	{
		printf( "WARNING: Attempt to load library \"%s\"\n", filename );
		return 1;
	}

	env->retcode = newCompleteException( env, InternalError, "Can't get value of java.lang.String" );
	return 0;
}
#endif /* JDK 1.1 */


/**
 * Enables/Disables tracing of byte-code instructions.  Passing true enables a
 * detail instruction trace, if the VM supports this feature.  Passing false
 * turns the trace off again.  Where the output goes is system dependant.
 *
 * This just sets out instruction trace flag (can be set with -i).
 *
 * @since   JDK1.0
 *
 * public native void traceInstructions(boolean on);
 */
void Java_java_lang_Runtime_traceInstructions( JNIEnv* env, jobject self, jboolean on )
{
#ifndef OPTIMIZED
	instruction_debug = on;
#endif /* OPTIMIZED */
}


/**
 * Enables/Disables tracing of all method calls.  Passing true enables a
 * trace of the entry and exit of all methods executed, if the VM supports this
 * feature.  Passing false turns the trace off again.
 *
 * This just sets our method_debug flag (can be set with -m).
 *
 * @since   JDK1.0
 *
 * public native void traceMethodCalls(boolean on);
 */
void Java_java_lang_Runtime_traceMethodCalls( JNIEnv* env, jobject self, jboolean on )
{
#ifdef _DEBUG
	method_debug = on;
#endif /* _DEBUG */
}
