/******************************************************************************
 *
 * Filename: sun_misc_Signal.c
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  6th May 2003
 *
 * Purpose:  Native method implementations for sun.misc.Signal.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <signal.h>

#include "../engine/config.h"

#include "sun_misc_Unsafe.h"

#include "../engine/util.h"


/* JDK 1.4
+   private static native int findSignal(java.lang.String);
-   private static native long handle0(int, long);
    private static native void raise0(int);
*/


#if JDK_VERSION >= 140
/**
 * ???Find the signal with the given name and return its signal number???
 *
 * private static native int findSignal( java.lang.String str );
 */
jint Java_sun_misc_Signal_findSignal( JNIEnv* env, jclass staticself, jobject str )
{
	char buf[255];

	if( extractFromJavaLangString( buf, 255, str ) )
	{
		if( strcmp( "INT", buf ) == 0 )
			return SIGINT;
		if( strcmp( "TERM", buf ) == 0 )
			return SIGTERM;
#ifndef WIN32
		if( strcmp( "HUP", buf ) == 0 )
			return SIGHUP;
#endif /* WIN32 */

		{
		char msg[255];
		sprintf( msg, "Unknown signal name \"%s\"", buf );
		env->retcode = newCompleteException( env, InternalError, msg );
		}
		return 0;
	}

	env->retcode = newCompleteException( env, InternalError, "Unable to extract signal name" );
	return 0;
}


/**
 * ???Find the signal with the given name and return its signal number???
 *
 * private static native long handle0( int sig, long aLong );
 */
jlong Java_sun_misc_Signal_handle0( JNIEnv* env, jclass staticself, jint sig, jlong aLong )
{
	printf( "WARNING: sun.misc.Signal.handle0( %d, %d ) dummied.\n", sig, aLong );

	if( aLong == (jlong) SIG_DFL )
		printf( "Request to set signal %d to SIG_DFL\n", sig );
	else if( aLong == (jlong) SIG_IGN )
		printf( "Request to set signal %d to SIG_IGN\n", sig );

	return 0;
}
#endif /* JDK 1.4 */
