/******************************************************************************
 *
 * Filename: java_util_zip_ZipEntry.h
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  10th May 2003
 *
 * Purpose:  Native method implementations for java.util.zip.ZipEntry.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#include <stdio.h>

#include "../engine/config.h"

#include "java_util_zip_ZipEntry.h"

#include "../engine/engine.h"
#include "../engine/util.h"
#include "../engine/globals.h"

#include "java_util_zip_ZipFile.h"


/* Methods: JDK1.4 (no native methods on JDK1.1.7A)
    private static native void initIDs();
    private native void initFields(long);
*/

/* Fields: JDK1.4 and JDK1.1.7A
	java.lang.String name;		 (1)
    long time;					 (2)
    long crc;					 (3)
    long size;					 (4)
    long csize;					(10)
    int method;                  (5)
    byte extra[];				 (6)
    java.lang.String comment;    (7)
    int flag;					 (8)
    int version;				 (9)
    long offset;				(11)
*/


#if JDK_VERSION >= 140
/**
 * private native void initFields(long);
 */
void Java_java_util_zip_ZipEntry_initFields( JNIEnv* env, jobject self, jlong handle )
{
	int index;
	ObjRef *name;
	int32 nameOffset, timeOffset, crcOffset, sizeOffset, csizeOffset, methodOffset;
	int32 extraOffset, commentOffset, flagOffset, versionOffset, offsetOffset;

	/*printf( "java.util.zip.ZipEntry.initFields( %d )\n", (int32) handle );*/

	/* Convert handle to an index into openentriesv */
	index = (int) handle - MAX_ZIPFILES;

	/* Check that index is valid and that it refers to an entry */
	if( (index < 0) || (index >= openentriesc) || !openentriesv[index] )
	{
		env->retcode = newCompleteException( env, InternalError, "Invalid ZipEntry handle" );
		return;
	}

	nameOffset    = getFieldOffset( self->cf, "name", "Ljava/lang/String;" );
	timeOffset    = getFieldOffset( self->cf, "time", "J" );
	crcOffset     = getFieldOffset( self->cf, "crc", "J" );
	sizeOffset    = getFieldOffset( self->cf, "size", "J" );
	csizeOffset   = getFieldOffset( self->cf, "csize", "J" );
	methodOffset  = getFieldOffset( self->cf, "method", "I" );
	extraOffset   = getFieldOffset( self->cf, "extra", "[B" );
	commentOffset = getFieldOffset( self->cf, "comment", "Ljava/lang/String;" );
	flagOffset    = getFieldOffset( self->cf, "flag", "I" );
	versionOffset = getFieldOffset( self->cf, "version", "I" );
	offsetOffset  = getFieldOffset( self->cf, "offset", "J" );

	if( (nameOffset < 0) || (timeOffset < 0) || (crcOffset < 0) || (sizeOffset < 0)
		|| (csizeOffset < 0) || (methodOffset < 0) || (extraOffset < 0) || (commentOffset < 0)
		|| (flagOffset < 0) || (versionOffset < 0) || (offsetOffset < 0) )
	{
		env->retcode = newCompleteException( env, InternalError, "Field(s) missing from java.util.zip.ZipEntry" );
		return;
	}

	name = newJavaLangStringFromBytes( openentriesv[index]->local.zipname, openentriesv[index]->local.zipfnln );
	self->data[nameOffset] = (u4) name;
	addToList( name, &objectPool );

	self->data[timeOffset]    = openentriesv[index]->local.ziptime;
	self->data[crcOffset]     = openentriesv[index]->local.zipcrc;
	self->data[sizeOffset]    = openentriesv[index]->local.zipuncmp;
	self->data[csizeOffset]   = openentriesv[index]->local.zipsize;
	self->data[methodOffset]  = openentriesv[index]->local.zipmthd;
	self->data[flagOffset]    = openentriesv[index]->local.zipgenflg;
	self->data[versionOffset] = openentriesv[index]->local.zipver;
	self->data[offsetOffset]  = openentriesv[index]->zipofst + 26
		+ openentriesv[index]->local.zipfnln
		+ openentriesv[index]->local.zipxtraln
		/* + openentriesv[index]->local.zipcmntln */;
}


/**
 * private static native void initIDs();
 */
void Java_java_util_zip_ZipEntry_initIDs( JNIEnv* stp, jclass staticself )
{
	/* Do Nothing! */
}
#endif /* JDK 1.4 */
