/******************************************************************************
 *
 * Filename: java_util_zip_ZipFile.h
 *
 * Author:   Robert Thornburrow
 * Project:  JavaVM
 * Created:  10th May 2003
 *
 * Purpose:  Native method implementations for java.util.zip.ZipFile.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

#ifndef _java_util_zip_ZipFile_h

#define _java_util_zip_ZipFile_h


#include "../engine/jni.h"

#include "../engine/zipfile.h"


#ifdef __cplusplus
extern "C" {
#endif

#if JDK_VERSION >= 140

#define MAX_ZIPFILES 32

/* Structure for recording open zip files */
typedef struct _OpenZipFile
{
	const char* filename;	/* File name of zip file */
	FILE*		f;			/* File handle */
	long		entries;	/* Number of entries in the zip file */
	struct _Hashtable*		cache;	/* A cache of the Central Directory entries */
	struct _ZipCacheEntry*	index;  /* A sequential list of ZipCacheEntries */
} OpenZipFile;

/* Structure for recording open zip file entries */
typedef struct _OpenZipEntry
{
	ZipLocalFile local;		/* Local file header */
	u4			 zipofst;	/* The offset of the entry into the zip file */
} OpenZipEntry;

/* List of open zip files */
extern OpenZipFile* openzipsv[];
extern int openzipsc;

/* List of open zip file entries */
extern OpenZipEntry* openentriesv[];
extern int openentriesc;


extern void    Java_java_util_zip_ZipFile_close( JNIEnv* env, jclass staticself, jlong handle );
extern void    Java_java_util_zip_ZipFile_freeEntry( JNIEnv* env, jclass staticself, jlong hzip, jlong hentry );
extern jint    Java_java_util_zip_ZipFile_getCSize( JNIEnv* env, jclass staticself, jlong hentry );
extern jlong   Java_java_util_zip_ZipFile_getEntry( JNIEnv* env, jclass staticself, jlong handle, jobject name );
extern jint    Java_java_util_zip_ZipFile_getMethod( JNIEnv* env, jclass staticself, jlong hentry );
extern jlong   Java_java_util_zip_ZipFile_getNextEntry( JNIEnv* env, jclass staticself, jlong handle, jint anInt );
extern jint    Java_java_util_zip_ZipFile_getSize( JNIEnv* env, jclass staticself, jlong hentry );
extern jint    Java_java_util_zip_ZipFile_getTotal( JNIEnv* env, jclass staticself, jlong handle );
extern jobject Java_java_util_zip_ZipFile_getZipMessage( JNIEnv* env, jclass staticself, jlong handle );
extern void    Java_java_util_zip_ZipFile_initIDs( JNIEnv* env, jclass staticself );
extern jlong   Java_java_util_zip_ZipFile_open( JNIEnv* env, jclass staticself, jobject file, jint mode, jlong mtime );
extern jint    Java_java_util_zip_ZipFile_read( JNIEnv* env, jclass staticself, jlong arg1, jlong arg2, jint arg3, jobject b, jint arg5, jint arg6 );
#endif /* JDK 1.4 */


#ifdef __cplusplus
}
#endif

#endif /* _java_util_zip_ZipFile_h */
